import { PrimitiveType, FormDefinitionAny, TypedFormControlBase } from '../model';
import { AbstractControlOptions, AsyncValidatorFn, FormArray, FormControl, FormGroup, ValidatorFn, ValidationErrors } from '@angular/forms';
import { Observable } from 'rxjs';
export declare class TypedFormControl<T> extends FormControl implements TypedFormControlBase {
    readonly formDefinition?: FormDefinitionAny<T> | undefined;
    constructor(formState?: T, validatorOrOpts?: ValidatorFn | ValidatorFn[] | AbstractControlOptions | null, asyncValidator?: AsyncValidatorFn | AsyncValidatorFn[] | null, formDefinition?: FormDefinitionAny<T> | undefined);
    get typedValue(): T;
    get typedValueChanges(): Observable<T>;
    get errorsWithAliasPath(): ValidationErrors | null;
    get parentTypedControl(): TypedFormControlBase;
    patchValue(value: T, options?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
        emitModelToViewChange?: boolean;
        emitViewToModelChange?: boolean;
    }): void;
    setValue(value: T, options?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
        emitModelToViewChange?: boolean;
        emitViewToModelChange?: boolean;
    }): void;
    reset(formState?: T, options?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    private emitIfRequired;
    markAsTouched(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    markAsUntouched(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    markAsDirty(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    markAsPristine(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
}
declare type TypedFormGroupChildInternal<F, K extends keyof F> = F[K] extends PrimitiveType ? TypedFormControl<F[K]> : F[K] extends (infer E)[] ? TypedFormArray<E> : TypedFormGroup<F[K]>;
declare type TypedFormGroupControlsInternal<F> = {
    [K in keyof F]: TypedFormGroupChildInternal<F, K>;
};
export declare class TypedFormGroup<F> extends FormGroup implements TypedFormControlBase {
    readonly formDefinition?: FormDefinitionAny<F> | undefined;
    constructor(controls: TypedFormGroupControlsInternal<F>, validatorOrOpts?: ValidatorFn | ValidatorFn[] | AbstractControlOptions | null, asyncValidator?: AsyncValidatorFn | AsyncValidatorFn[] | null, formDefinition?: FormDefinitionAny<F> | undefined);
    get typedValue(): F;
    get typedRawValue(): F;
    get typedValueChanges(): Observable<F>;
    get typedRawValueChanges(): Observable<F>;
    get typedControls(): TypedFormGroupControlsInternal<F>;
    get errorsWithAliasPath(): ValidationErrors | null;
    get parentTypedControl(): TypedFormControlBase;
    typedGet<K extends keyof F>(key: K): TypedFormGroupChildInternal<F, K>;
    typedGetCustomField<K extends keyof F>(key: K): TypedFormControl<F[K]>;
    patchValue(value: Partial<F>, options?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
        emitModelToViewChange?: boolean;
        emitViewToModelChange?: boolean;
    }): void;
    /** @ts-expect-error For the moment we need to omit override modifier because of parameter type incompatibility */
    setValue(value: F, options?: {
        onlySelf?: boolean | undefined;
        emitEvent?: boolean | undefined;
        emitModelToViewChange?: boolean | undefined;
        emitViewToModelChange?: boolean | undefined;
    }): void;
    reset(formState?: F, options?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    private emitIfRequired;
    markAsTouched(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    markAsUntouched(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    markAsDirty(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    markAsPristine(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
}
declare type TypedFormControlOrGroupArrayInternal<T> = T extends PrimitiveType ? TypedFormControl<T> : TypedFormGroup<T>;
export declare class TypedFormArray<T> extends FormArray implements TypedFormControlBase {
    readonly formDefinition?: FormDefinitionAny<T> | undefined;
    constructor(controls: TypedFormControlOrGroupArrayInternal<T>[], validatorOrOpts?: ValidatorFn | ValidatorFn[] | AbstractControlOptions | null, asyncValidator?: AsyncValidatorFn | AsyncValidatorFn[] | null, formDefinition?: FormDefinitionAny<T> | undefined);
    get typedValue(): T[];
    get typedRawValue(): T[];
    get typedControls(): TypedFormControlOrGroupArrayInternal<T>[];
    get typedValueChanges(): Observable<T[]>;
    get typedRawValueChanges(): Observable<T[]>;
    get errorsWithAliasPath(): ValidationErrors | null;
    get parentTypedControl(): TypedFormControlBase;
    typedAt(index: number): TypedFormControlOrGroupArrayInternal<T> | null;
    typedAtCustomField(index: number): TypedFormControl<T> | null;
    patchValue(value: T[], options?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
        emitModelToViewChange?: boolean;
        emitViewToModelChange?: boolean;
    }): void;
    setValue(value: T[], options?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
        emitModelToViewChange?: boolean;
        emitViewToModelChange?: boolean;
    }): void;
    reset(formState?: T[], options?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    private emitIfRequired;
    markAsTouched(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    markAsUntouched(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    markAsDirty(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
    markAsPristine(opts?: {
        onlySelf?: boolean;
        emitEvent?: boolean;
    }): void;
}
export {};
